/*
 * tree.h - simple multidimensional tree
 *
 */

#ifndef TREE_H_
#define TREE_H_

struct tree_node {
	struct tree_node	*parent;
	struct tree_node	*next;
	struct tree_node	*child;
	void			*data;
};

typedef struct tree_node* tree_node_t;
typedef int (*treecallback_t)(tree_node_t node, void *opaque);

/* tree_node_new - create new tree node
 * Returns new tree node.
 */
tree_node_t tree_node_new();

/* tree_node_free - free one node
 * node: node to free
 */
void tree_node_free(tree_node_t *node);

/* tree_node_free_path - free resources of the node and it's descendents
 * node: node to free
 */
void tree_node_free_path(tree_node_t *node);

/* tree_node_add - add node to the tree
 * parent: parent node
 * node: node to add
 * where: 0 - add as the last child, 1 - add as the first child
 */
void tree_node_add(tree_node_t parent, tree_node_t node, int where);

/* tree_node_reverse - reverse child nodes
 * root: root node
 */
void tree_node_reverse(tree_node_t root);

/* tree_node_del - delete node from the tree
 * node: node to delete
 * Returns deleted tree node.
 */
tree_node_t tree_node_del(tree_node_t node);

/* tree_node_add_sibling - add node as sibling node
 * sibling: sibling node
 * node: node to add
 * where: 0 - add as the left sibling, 1 - add as the right sibling
 */
void tree_node_add_sibling(tree_node_t sibling, tree_node_t node, int where);

/* find node in the tree at the same or ancestory paths */

/* tree_node_find - find node in the tree
 * root: use root as the root of the search
 * data: data to search for
 * search_children: 0 - do not search in the child nodes, 1 - search child nodes
 * compare: function to compare two nodes or NULL
 */
tree_node_t tree_node_find(tree_node_t root, void *data, int search_children,
		int (*compare)(const void *a, const void *b));

/* tree_node_find_child - find node among children
 *
 */
tree_node_t tree_node_find_child(tree_node_t parent, void *data,
		int (*compare)(const void *node_data, const void *data));

/* tree_node_traverse - traverse the tree and call callback on each node
 * root: use root as the root of the traversal
 * callback: callback to call on each node
 * child_nodes: traverse child nodes or not (TRUE or FALSE)
 * opaque: opaque pointer supplied to callback
 */
void tree_node_traverse(tree_node_t root, treecallback_t callback,
		int child_nodes, void *opaque);

#endif /* TREE_H_ */
